#
# Lightweight Zope startup
#
# Fast Zope startup is achieved by not installing (m)any 
# products. If your tests require a product you must 
# install it yourself using installProduct().
#
# Typically used as in
#
#   import ZopeLite as Zope
#   Zope.installProduct('SomeProduct')
#   app = Zope.app()
#

__version__ = '0.5.2'

import os, sys, time

def _print(msg):
    sys.stdout.write(msg)
    sys.stdout.flush()

_print('Loading Zope, please stand by ')
_start = time.time()

import ZODB
_print('.')

import Globals  
_print('.')

import OFS.Application  
import App.ProductContext

# Avoid expensive product import
def _null_import(): pass    
OFS.Application.import_products = _null_import

# Avoid expensive product installation
def _null_init(app): pass     
OFS.Application.initialize = _null_init

# Avoid expensive help registration
def _null_register_topic(self,id,topic): pass
App.ProductContext.ProductContext.registerHelpTopic = _null_register_topic
def _null_register_title(self,title): pass
App.ProductContext.ProductContext.registerHelpTitle = _null_register_title
def _null_register_help(self,directory='',clear=1,title_re=None): pass  
App.ProductContext.ProductContext.registerHelp = _null_register_help

# Make sure to use a temporary client cache
if os.environ.get('ZEO_CLIENT'): del os.environ['ZEO_CLIENT']

# Load Zope
import Zope
_print('.')

from OFS.Application import get_folder_permissions, get_products, install_product
import Products, OFS.Folder

_theApp = Zope.app()
_installedProducts = {}

def hasProduct(name):
    '''Tests if a Product can be found along Products.__path__'''
    return name in [n[1] for n in get_products()]

def installProduct(name, quiet=0):
    '''Installs a Zope Product'''
    _start = time.time()
    app = _theApp
    meta_types = []
    if not _installedProducts.has_key(name):
        for priority, product_name, index, product_dir in get_products(): 
            if product_name == name:
                if not quiet: _print('Installing %s ... ' % product_name)
                install_product(app, product_dir, product_name, meta_types, get_folder_permissions())
                _installedProducts[product_name] = 1
                Products.meta_types = Products.meta_types + tuple(meta_types)
                Globals.default__class_init__(OFS.Folder)
                if not quiet: _print('done (%.3fs)\n' % (time.time() - _start))
                break

# The ZEO_CLIENT variable does two unrelated things :-P
if os.environ.get('NO_PRODUCT_LOAD'):
    os.environ['ZEO_CLIENT'] = '1'

# XXX: Pay no attention to that man behind the curtain!
_s=time.time(); _d=(_s-_start)/4
_theApp.Control_Panel
if (time.time()-_s)>_d: _print('.')

installProduct('PluginIndexes', 1)  # Must install first
installProduct('OFSP', 1)
#installProduct('ExternalMethod', 1)
#installProduct('ZSQLMethods', 1)
#installProduct('ZGadflyDA', 1)
#installProduct('MIMETools', 1)
#installProduct('MailHost', 1)

# So people can use ZopeLite.app()
app = Zope.app

_print(' done (%.3fs)\n' % (time.time() - _start))

