import unittest
from sys import path
path.append('/usr/lib/zope/lib/python/Products')
from IcubeZTester.zopetester import zopetester
import ZopeLite as Zope
Zope.installProduct('PythonScripts')

class zopeTestCase(unittest.TestCase):

    def setUp(self):
        self.zope = Zope.app()
        self.zope.manage_addProduct['IcubeZTester'].manage_addIcubeZTester('ZT')
        self.zt = getattr(self.zope, 'ZT')

    def tearDown(self):
        get_transaction().abort()
        self.zope._p_jar.close()

    def testPass(self):
        pass

    def testId(self):
        assert self.zt.id == 'ZT'

    def testGetAllTests(self):
        result = self.zt.getAllTests()
        assert result == []
        self.zope.manage_addProduct['PythonScripts'].manage_addPythonScript('test01')
        self.zope.manage_addProduct['PythonScripts'].manage_addPythonScript('test02')
        self.zope.manage_addProduct['PythonScripts'].manage_addPythonScript('non_test')
        result = self.zt.getAllTests()
        assert len(result) == 2, "ne ha trovati %d" % len(result)

    def testGetRemoteTests2(self):
        self.zt.addRemoteTest('')
        self.zt.addRemoteTest('doesNotExits')
        self.zt.addRemoteTest(' ')
        assert len(self.zt.getAllTests()) == 0, \
               "ne ha trovati %d" % len(self.zt.getAllTests())
        

    def testGetRemoteTests1(self):
        self.zope.manage_addProduct['OFSP'].manage_addFolder('FolderA')
        folder = self.zope.FolderA
        folder.manage_addProduct['PythonScripts'].manage_addPythonScript('test01')
        folder.manage_addProduct['OFSP'].manage_addFolder('FolderB')
        self.zt.addRemoteTest('FolderA/test01')
        self.zt.addRemoteTest('FolderA/FolderB')
        self.zt.addRemoteTest('FolderA/test01')
        assert len(self.zt.getAllTests()) == 1, \
               "ne ha trovati %d" % len(self.zt.getAllTests())
        folder.FolderB.manage_addProduct['PythonScripts'].manage_addPythonScript('test02')
        assert len(self.zt.getAllTests()) == 2, \
               "ne ha trovati %d" % len(self.zt.getAllTests())
        self.zope.manage_addProduct['OFSP'].manage_addFolder('FolderC')
        self.zope.FolderC.manage_addProduct['PythonScripts'].manage_addPythonScript('test01')
        self.zope.FolderC.manage_addProduct['IcubeZTester'].manage_addIcubeZTester('zt2')
        self.zt.addRemoteTest('FolderC/zt2')
        assert len(self.zt.getAllTests()) == 3, \
               "ne ha trovati %d" % len(self.zt.getAllTests())

    def createPyScripts(self):
        self.zope.manage_addProduct['PythonScripts'].manage_addPythonScript('test01')
        self.zope.test01.ZPythonScript_edit(params='', body='pass')
        self.zope.manage_addProduct['PythonScripts'].manage_addPythonScript('test02')
        self.zope.test02.ZPythonScript_edit(params='', body='assert 1==2, "some string"')
        self.zope.manage_addProduct['PythonScripts'].manage_addPythonScript('test03')
        self.zope.test03.ZPythonScript_edit(params='', body='thisShouldError')
        self.zope.manage_addProduct['PythonScripts'].manage_addPythonScript('non_test')
        return self.zope.test01, self.zope.test02, self.zope.test03
        
    def testRunAllTests1(self):
        t1, t2, t3 = self.createPyScripts()
        self.zt.setSkipPass(1)
        result = self.zt.runAllTests()
        expected = {'pass': [t1],
                    'fail': [t2],
                    'error': [t3]}
        assert result['pass'] == expected['pass'], result
        assert result['fail'] == expected['fail'], result
        assert result['error'] == expected['error'], result
        assert len(result['table']) == 2

    def testRunAllTests2(self):
        t1, t2, t3 = self.createPyScripts()
        self.zt.setSkipPass(0)
        assert len(self.zope.objectValues('Script (Python)')) == 4
        assert t1 in self.zope.objectValues('Script (Python)')
        assert len(self.zt.getAllTests()) == 3
        result = self.zt.runAllTests()
        expected = {'pass': [t1],
                    'fail': [t2],
                    'error': [t3]}
        assert result['pass'] == expected['pass'], result
        assert result['fail'] == expected['fail'], result
        assert result['error'] == expected['error'], result
        assert len(result['table']) == 3
        for item in result['table']:
            assert item.has_key('test')
            assert item.has_key('result')
            assert item.has_key('exception_type')
            assert item.has_key('short_info')
            assert item.has_key('long_info')
       
