from AccessControl import ClassSecurityInfo
from Globals import InitializeClass, DTMLFile
from OFS.SimpleItem import SimpleItem

PASS_VALUE = 'pass'
FAIL_VALUE = 'fail'
ERROR_VALUE = 'error'

def manage_addIcubeZTester(self, id, title='', REQUEST=None):
    """ adds a zopetester """
    w = zopetester(id, title)
    self._setObject(id, w)
    if REQUEST is not None:
        REQUEST.RESPONSE.redirect('manage_main')

manage_addIcubeZTesterForm = DTMLFile('dtml/manage_addIcubeZTesterForm', globals())

class zopetester(SimpleItem):
    """ The zopetester object """

    def __init__(self, id, title=''):
        self.id = id
        self.title = title
        self._remote_tests = []
        self._skip_pass = 1

    meta_type = "Icube ZTester"

    manage_options = ({'action':'suite_settings', 'label':'Settings'},
                      {'action':'run_tests', 'label':'Run Tests'}) + \
                     SimpleItem.manage_options
    run_tests = DTMLFile('dtml/runTests', globals())
    suite_settings = DTMLFile('dtml/suiteSettings', globals())

    security = ClassSecurityInfo()

    def runScript(self, script):
        """ doc """
        def getExceptInfo():
            import sys, traceback
            type, value, tb = sys.exc_info()
            return str(type), value, traceback.format_tb(tb)
        try:
            script()
        except AssertionError, ae:
            get_transaction().abort()
            t, v, tb = getExceptInfo()
            return (FAIL_VALUE, t, v, tb)
        except:
            get_transaction().abort()
            t, v, tb = getExceptInfo()
            return (ERROR_VALUE, t, v, tb)
        else:
            get_transaction().abort()
            return (PASS_VALUE, '', '', [])

    def addRemoteTest(self, test):
        """ """
        self._remote_tests.append(test)
        self._p_changed = 1

    def getRemoteTests(self):
        """ """
        return self._remote_tests

    def setRemoteTests(self, test_list, REQUEST=None):
        """ """
        self._remote_tests = test_list
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(REQUEST.HTTP_REFERER)

    def getAllTests(self):
        """ returns all the tests """
        def appendScript(script, list):
            if script.id[0:4]=='test' and script not in list:
                list.append(script)
        container = self.aq_parent
        test_list = []
        for script in container.objectValues('Script (Python)'):
            appendScript(script, test_list)
        for remote_test in self._remote_tests:
            ref_object = None
            try:
                if remote_test: #we get a bad error if remote_test is ''
                    ref_object = self.restrictedTraverse(remote_test)
            except:
                pass
            if ref_object:
                if ref_object.meta_type == 'Script (Python)':
                    appendScript(ref_object, test_list)
                elif ref_object.meta_type == 'Folder':
                    for script in ref_object.objectValues('Script (Python)'):
                        appendScript(script, test_list)
                elif ref_object.meta_type == self.meta_type:
                    for script in ref_object.getAllTests():
                        appendScript(script, test_list)
        return test_list

    def colorCode(self, code):
        """ string """
        color_codes = {PASS_VALUE:'#66DD66',
                       FAIL_VALUE:'#DDDD66',
                       ERROR_VALUE:'#DD6666',
                       }
        return color_codes[code]

    def setSkipPass(self, value, REQUEST=None):
        """ sets the value of skip_pass """
        self._skip_pass = value
        if REQUEST:
            self.REQUEST.RESPONSE.redirect(REQUEST.HTTP_REFERER)

    def getSkipPass(self):
        """ gets the value of skip pass.
        For backward compatibility if skip_pass is not present
        it is set to 1 and returned"""
        if not hasattr(self, '_skip_pass'):
            self.setSkipPass(1)
        return self._skip_pass

    def runAllTests(self):
        """ stringhina """
        import DateTime
        start_time = DateTime.DateTime()
        tests = self.getAllTests()
        result = {PASS_VALUE: [],
                  FAIL_VALUE: [],
                  ERROR_VALUE: [],
                  'table':[]}
        for test in tests:
            [res, ex_type, s_info, l_info] = self.runScript(test)
            result[res].append(test)
            if res!=PASS_VALUE or not self.getSkipPass():
                result['table'].append({'test':test,
                                        'result':res,
                                        'exception_type':ex_type,
                                        'short_info':s_info,
                                        'long_info':l_info})
        end_time = DateTime.DateTime()
        result['total_time'] = float(end_time.millis() - start_time.millis()) / 1000
        return result
    
InitializeClass(zopetester)
